﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using ThreeJS.Camera;
using ThreeJS.Mesh;
using ThreeJS.Utils;
using Windows.UI;
using Windows.UI.Input;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

namespace Examples
{
    public partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();
            Viewport3D_1.Viweport3D_Ready += Viewport3D_1_Viweport3D_Ready;
        }

        private void Viewport3D_1_Viweport3D_Ready(object sender, RoutedEventArgs e)
        {
            ShowCubes(sender, e);
        }


        private void ShowTrianges(object sender, RoutedEventArgs e)
        {
            PerspectiveCamera persCam = new PerspectiveCamera() { fov = 80, aspect = Viewport3D_1.Width / Viewport3D_1.Height, near = 1, far = 1000 };
            persCam.position.X = 0;
            persCam.position.Y = 0;
            persCam.position.Z = 500;

            Random r = new Random();
            Model3DGroup grp = new Model3DGroup();

            for (int loop = 0; loop < 20; loop++)
            {
                MeshGeometry3D mesh = new MeshGeometry3D();
                for (int i = 0; i < 3; i++)
                {
                    mesh.Positions.Add(new Point3D(-200 + 400 * r.NextDouble(), -200 + 400 * r.NextDouble(), -200 + 400 * r.NextDouble()));
                    mesh.TriangleIndices.Add(i);
                }

                Color color = Color.FromArgb(255, (byte)(255 + r.NextDouble()), (byte)(255 + r.NextDouble()), (byte)(255 + r.NextDouble()));
                Material material = new DiffuseMaterial(color);
                GeometryModel3D gm = new GeometryModel3D((Geometry3D)mesh, (Material)material);
                grp.Children.Add(gm);
            }

            Viewport3D_1.Display(persCam, grp);

            Viewport3D_1.Render();

        }

        private void ShowSpheres(object sender, RoutedEventArgs e)
        {
            {

                PerspectiveCamera persCam = new PerspectiveCamera() { fov = 80, aspect = Viewport3D_1.Width / Viewport3D_1.Height, near = 1, far = 1000 };
                persCam.position.X = 0;
                persCam.position.Y = 0;
                persCam.position.Z = 500;

                MeshHelper mh = new MeshHelper();

                Model3DGroup grp = new Model3DGroup();

                {
                    MeshGeometry3D mesh = mh.Sphere(new Point3D(-100, -100, -100), 70, 20);
                    Material material = new DiffuseMaterial(Colors.Red);
                    GeometryModel3D gm = new GeometryModel3D((Geometry3D)mesh, (Material)material);
                    grp.Children.Add(gm);
                }

                {
                    MeshGeometry3D mesh = mh.Sphere(new Point3D(100, 100, 100), 70, 20);
                    Material material = new DiffuseMaterial(Colors.Green);
                    GeometryModel3D gm = new GeometryModel3D((Geometry3D)mesh, (Material)material);
                    grp.Children.Add(gm);
                }

                Viewport3D_1.Display(persCam, grp);

                Viewport3D_1.Render();
            }

        }

        private void ShowCubes(object sender, RoutedEventArgs e)
        {

            PerspectiveCamera persCam = new PerspectiveCamera() { fov = 80, aspect = Viewport3D_1.Width / Viewport3D_1.Height, near = 1, far = 1000 };
            persCam.position.X = 0;
            persCam.position.Y = 0;
            persCam.position.Z = 500;

            MeshHelper mh = new MeshHelper();

            Model3DGroup grp = new Model3DGroup();

            {
                MeshGeometry3D mesh = mh.Box(150, 150, 150, new Point3D(-100, -100, 100), 0);
                Material material = new DiffuseMaterial(Colors.Red);
                GeometryModel3D gm = new GeometryModel3D((Geometry3D)mesh, (Material)material);
                grp.Children.Add(gm);
            }

            {
                MeshGeometry3D mesh = mh.Box(150, 150, 150, new Point3D(100, 100, 100), 0);
                Material material = new DiffuseMaterial(Colors.Green);
                GeometryModel3D gm = new GeometryModel3D((Geometry3D)mesh, (Material)material);
                grp.Children.Add(gm);
            }

            {
                MeshGeometry3D mesh = mh.Box(150, 150, 150, new Point3D(-100, 100, 100), 0);
                Material material = new DiffuseMaterial(Colors.Blue);
                GeometryModel3D gm = new GeometryModel3D((Geometry3D)mesh, (Material)material);
                grp.Children.Add(gm);
            }

            {
                MeshGeometry3D mesh = mh.Box(150, 150, 150, new Point3D(100, -100, 100), 0);
                Material material = new DiffuseMaterial(Colors.Yellow);
                GeometryModel3D gm = new GeometryModel3D((Geometry3D)mesh, (Material)material);
                grp.Children.Add(gm);
            }

            {
                MeshGeometry3D mesh = mh.Box(150, 150, 150, new Point3D(-100, -100, -100), 0);
                Material material = new DiffuseMaterial(Colors.Orange);
                GeometryModel3D gm = new GeometryModel3D((Geometry3D)mesh, (Material)material);
                grp.Children.Add(gm);
            }

            {
                MeshGeometry3D mesh = mh.Box(150, 150, 150, new Point3D(100, 100, -100), 0);
                Material material = new DiffuseMaterial(Colors.Purple);
                GeometryModel3D gm = new GeometryModel3D((Geometry3D)mesh, (Material)material);
                grp.Children.Add(gm);
            }

            {
                MeshGeometry3D mesh = mh.Box(150, 150, 150, new Point3D(-100, 100, -100), 0);
                Material material = new DiffuseMaterial(Colors.SlateBlue);
                GeometryModel3D gm = new GeometryModel3D((Geometry3D)mesh, (Material)material);
                grp.Children.Add(gm);
            }

            {
                MeshGeometry3D mesh = mh.Box(150, 150, 150, new Point3D(100, -100, -100), 0);
                Material material = new DiffuseMaterial(Colors.Pink);
                GeometryModel3D gm = new GeometryModel3D((Geometry3D)mesh, (Material)material);
                grp.Children.Add(gm);
            }

            Viewport3D_1.Display(persCam, grp);

            Viewport3D_1.Render();
        }

        bool cPointerPressed = false;
        PointerPoint cPointerPoint;
        private void V3D_PointerPressed(object sender, Windows.UI.Xaml.Input.PointerRoutedEventArgs e)
        {
            cPointerPoint = e.GetCurrentPoint(Viewport3D_1);
            //CapturePointer(e.Pointer);
            cPointerPressed = true;
        }

        private void V3D_PointerReleased(object sender, Windows.UI.Xaml.Input.PointerRoutedEventArgs e)
        {
            //ReleasePointerCapture(e.Pointer);
            cPointerPressed = false;
        }

        private void V3D_PointerMoved(object sender, Windows.UI.Xaml.Input.PointerRoutedEventArgs e)
        {
            if (cPointerPressed)
            {
                PointerPoint pointerPoint = e.GetCurrentPoint(Viewport3D_1);
                Viewport3D_1.Rotation.Y += (pointerPoint.Position.X - cPointerPoint.Position.X) * 0.02;
                cPointerPoint = pointerPoint;
                Viewport3D_1.Render();
            }
        }

    }
}
