﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ThreeJS.Mesh;

namespace ThreeJS.Utils
{
    public class Transform3DGroup
    {
        public Point3D Transform(Point3D pInput)
        {
            return pInput;
        }
    }

    public class MeshHelper
    {
        Transform3DGroup cPoleTansformationGroup = new Transform3DGroup();

        public MeshGeometry3D Sphere(Point3D pCenter, double pRadius, int pNumSegments)
        {
            int e;
            double segmentRad = Math.PI / 2 / (pNumSegments + 1);
            int numberOfSeparators = 4 * pNumSegments + 4;

            MeshGeometry3D mesh = new MeshGeometry3D();

            for (e = -pNumSegments; e <= pNumSegments; e++)
            {
                double r_e = pRadius * Math.Cos(segmentRad * e);
                double y_e = pRadius * Math.Sin(segmentRad * e);

                for (int s = 0; s <= (numberOfSeparators - 1); s++)
                {
                    double z_s = r_e * Math.Sin(segmentRad * s) * (-1);
                    double x_s = r_e * Math.Cos(segmentRad * s);
                    mesh.Positions.Add(cPoleTansformationGroup.Transform(new Point3D(pCenter.X + x_s, pCenter.Y + y_e, pCenter.Z + z_s)));
                }
            }

            mesh.Positions.Add(cPoleTansformationGroup.Transform(new Point3D(pCenter.X, pCenter.Y + pRadius, pCenter.Z)));
            mesh.Positions.Add(cPoleTansformationGroup.Transform(new Point3D(pCenter.X, pCenter.Y - pRadius, pCenter.Z)));

            for (e = 0; e < 2 * pNumSegments; e++)
            {
                for (int i = 0; i < numberOfSeparators; i++)
                {
                    mesh.TriangleIndices.Add(e * numberOfSeparators + (i + 1) %
                                        numberOfSeparators + numberOfSeparators);
                    mesh.TriangleIndices.Add(e * numberOfSeparators + i +
                                        numberOfSeparators);
                    mesh.TriangleIndices.Add(e * numberOfSeparators + i);

                    mesh.TriangleIndices.Add(e * numberOfSeparators + i);
                    mesh.TriangleIndices.Add(e * numberOfSeparators +
                                       (i + 1) % numberOfSeparators);
                    mesh.TriangleIndices.Add(e * numberOfSeparators + (i + 1) %
                                        numberOfSeparators + numberOfSeparators);
                }
            }

            for (int i = 0; i < numberOfSeparators; i++)
            {
                mesh.TriangleIndices.Add(e * numberOfSeparators + (i + 1) %
                                    numberOfSeparators);
                mesh.TriangleIndices.Add(numberOfSeparators * (2 * pNumSegments + 1));
                mesh.TriangleIndices.Add(e * numberOfSeparators + i);
            }

            for (int i = 0; i < numberOfSeparators; i++)
            {
                mesh.TriangleIndices.Add(i);
                mesh.TriangleIndices.Add((i + 1) % numberOfSeparators);
                mesh.TriangleIndices.Add(numberOfSeparators * (2 * pNumSegments + 1) + 1);
            }

            return mesh;
        }

        /// <summary>
        /// Create a 3D mesh representing a box
        /// </summary>
        /// <param name="pLength">Length of the box along the screen's x-axis.</param>
        /// <param name="pDepth">Depth of the box along the axis into the screen (deep+).</param>
        /// <param name="pHeight">Height of the box along the screen's y-axis (up+).</param>
        /// <param name="pCenterPt">The real world coordinate of the center of the box.</param>
        /// <param name="pRotation">The rotation of the box geometry about the y-axis through its center point.</param>
        /// <returns></returns>
        public MeshGeometry3D Box(
            double pLength,
            double pDepth,
            double pHeight,
            Point3D pCenterPt,
            double pRotation)
        {
            MeshGeometry3D mesh = new MeshGeometry3D();
            // X and Y vals on face at one end of length (-rotation direction)
            //// Edge closest to origin
            double s1x1 = pCenterPt.X - ((Math.Cos(pRotation) * (pLength / 2)) + (Math.Sin(pRotation) * (pDepth / 2)));
            double s1z1 = pCenterPt.Z - ((Math.Sin(pRotation) * (pLength / 2)) - (Math.Cos(pRotation) * (pDepth / 2)));
            //// Edge furthest from origin
            double s1x2 = pCenterPt.X - ((Math.Cos(pRotation) * (pLength / 2)) - (Math.Sin(pRotation) * (pDepth / 2)));
            double s1z2 = pCenterPt.Z - ((Math.Sin(pRotation) * (pLength / 2)) + (Math.Cos(pRotation) * (pDepth / 2)));
            // X and Y vals on face at other end of length
            //// Edge closest to origin
            double s2x1 = pCenterPt.X + ((Math.Cos(pRotation) * (pLength / 2)) - (Math.Sin(pRotation) * (pDepth / 2)));
            double s2z1 = pCenterPt.Z + ((Math.Sin(pRotation) * (pLength / 2)) + (Math.Cos(pRotation) * (pDepth / 2)));
            //// Edge furthest to origin
            double s2x2 = pCenterPt.X + ((Math.Cos(pRotation) * (pLength / 2)) + (Math.Sin(pRotation) * (pDepth / 2)));
            double s2z2 = pCenterPt.Z + ((Math.Sin(pRotation) * (pLength / 2)) - (Math.Cos(pRotation) * (pDepth / 2)));

            double y1 = pCenterPt.Y + (pHeight / 2);
            double y2 = pCenterPt.Y - (pHeight / 2);

            //BoxRotation br = new BoxRotation(pHorizontalOffset, pRotation, pVerticalOffset);

            mesh.Positions.Add(cPoleTansformationGroup.Transform(new Point3D(s2x1, y1, s2z1)));
            mesh.Positions.Add(cPoleTansformationGroup.Transform(new Point3D(s2x2, y1, s2z2)));
            mesh.Positions.Add(cPoleTansformationGroup.Transform(new Point3D(s1x2, y1, s1z2)));
            mesh.Positions.Add(cPoleTansformationGroup.Transform(new Point3D(s1x1, y1, s1z1)));
            mesh.Positions.Add(cPoleTansformationGroup.Transform(new Point3D(s2x1, y2, s2z1)));
            mesh.Positions.Add(cPoleTansformationGroup.Transform(new Point3D(s2x2, y2, s2z2)));
            mesh.Positions.Add(cPoleTansformationGroup.Transform(new Point3D(s1x2, y2, s1z2)));
            mesh.Positions.Add(cPoleTansformationGroup.Transform(new Point3D(s1x1, y2, s1z1)));

            //top
            mesh.TriangleIndices.Add(0);
            mesh.TriangleIndices.Add(1);
            mesh.TriangleIndices.Add(2);

            mesh.TriangleIndices.Add(2);
            mesh.TriangleIndices.Add(3);
            mesh.TriangleIndices.Add(0);

            //bottom
            mesh.TriangleIndices.Add(5);
            mesh.TriangleIndices.Add(4);
            mesh.TriangleIndices.Add(7);

            mesh.TriangleIndices.Add(7);
            mesh.TriangleIndices.Add(6);
            mesh.TriangleIndices.Add(5);

            //front
            mesh.TriangleIndices.Add(2);
            mesh.TriangleIndices.Add(1);
            mesh.TriangleIndices.Add(5);

            mesh.TriangleIndices.Add(5);
            mesh.TriangleIndices.Add(6);
            mesh.TriangleIndices.Add(2);

            //back
            mesh.TriangleIndices.Add(0);
            mesh.TriangleIndices.Add(3);
            mesh.TriangleIndices.Add(7);

            mesh.TriangleIndices.Add(7);
            mesh.TriangleIndices.Add(4);
            mesh.TriangleIndices.Add(0);

            //end1
            mesh.TriangleIndices.Add(3);
            mesh.TriangleIndices.Add(2);
            mesh.TriangleIndices.Add(6);

            mesh.TriangleIndices.Add(6);
            mesh.TriangleIndices.Add(7);
            mesh.TriangleIndices.Add(3);

            //end2
            mesh.TriangleIndices.Add(1);
            mesh.TriangleIndices.Add(0);
            mesh.TriangleIndices.Add(4);

            mesh.TriangleIndices.Add(4);
            mesh.TriangleIndices.Add(5);
            mesh.TriangleIndices.Add(1);

            return mesh;
        }
    }
}
