﻿using CSHTML5;
using CSHTML5.Native.Html.Controls;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ThreeJS.Camera;
using ThreeJS.Mesh;
using ThreeJS.Utils;
using Windows.UI;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media;

namespace ThreeJS
{

    public class Viewport3D : HtmlPresenter
    {

        // Remember whether the JS library was loaded in order not to load it twice:
        static bool JSLibraryWasLoaded;
        string id;

        public delegate void Viweport3D_ReadyDelegate(object sender, Windows.UI.Xaml.RoutedEventArgs e);
        public event Viweport3D_ReadyDelegate Viweport3D_Ready;

        public Viewport3D()
        {
            id = "3D_" + Guid.NewGuid().ToString("N");

            this.Html = "<canvas id=\"" + id + "\" name=\"" + id + "\" ></ canvas > ";
            this.Loaded += Viewport3D_Loaded;
            this.SizeChanged += Viewport3D_SizeChanged;
        }

        private void Viewport3D_SizeChanged(object sender, Windows.UI.Xaml.SizeChangedEventArgs e)
        {
            if (double.IsNaN(this.Width)) return;
            if (double.IsNaN(this.Height)) return;
            this.Html = "<canvas id=\"" + id + "\" name=\"" + id + "width = \"" + this.Width.ToString() + "\" height = \"" + this.Height.ToString() + "\" ></ canvas > ";
        }


        async void Viewport3D_Loaded(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            // Load the JavaScript libraries if not already loaded:
            if (!JSLibraryWasLoaded)
            {
                await Interop.LoadJavaScriptFile("ms-appx:///ThreeJS/JS/three.js");
                await Interop.LoadJavaScriptFile("ms-appx:///ThreeJS/JS/Projector.js");
                await Interop.LoadJavaScriptFile("ms-appx:///ThreeJS/JS/CanvasRenderer.js");
                await Interop.LoadJavaScriptFile("ms-appx:///ThreeJS/JS/Viewport3D.js");
                JSLibraryWasLoaded = true;
                if (Viweport3D_Ready != null) Viweport3D_Ready(this, e);
            }
        }

        public Point3D Rotation = new Point3D();

        public void Display(PerspectiveCamera pCamera, Model3DGroup pGroup)
        {
            if (!JSLibraryWasLoaded) return;

            StringBuilder json = new StringBuilder();
            json.Append("{\"camera\":");
            pCamera.ToJSON(json);
            json.Append(',');
            pGroup.ToJSON(json);
            json.Append('}');

            Interop.ExecuteJavaScript(@"V3D_targetCanvas = document.getElementById($0);", this.id);
            Interop.ExecuteJavaScript(@"V3D_targetWidth = $0; V3D_targetHeight = $1;", this.Width, this.Height);

            Interop.ExecuteJavaScript(@"V3D_params = JSON.parse($0);", json.ToString());
            Interop.ExecuteJavaScript(@"init();");
        }

        public void Render()
        {
            Interop.ExecuteJavaScript(@"V3D_sceneRotationX=$0; V3D_sceneRotationY=$1; V3D_sceneRotationZ=$2;", Rotation.X, Rotation.Y, Rotation.Z);
            Interop.ExecuteJavaScript(@"render();");
        }

    }
}
